#####################################################################################
###                                                                               ###
### These functions modify the partykit plotting functions to improve resolution  ###
### and readability.  Most of these functions make slight changes to the original ###
### functions, e.g., fontsize is increased or the terminal node adds the          ###
### probability of survival.  While the original partykit functions have ways to  ###
### modify the figures, creating our own plotting functions allow more control.   ###
###                                                                               ###
#####################################################################################

node_inner_fig1 <- function(obj) {
  meta <- obj$data
  nam <- names(obj)
  extract_label <- function(node){
    if (is.terminal(node)) return(rep.int("", 2L))
    varlab <- character_split(split_node(node), meta)$name
    return(c(varlab, ""))
  }
  maxstr <- function(node) {
    lab <- extract_label(node)
    klab <- if (is.terminal(node)) "" else unlist(lapply(kids_node(node), maxstr))
    lab <- c(lab, klab)
    lab <- unlist(lapply(lab, function(x) strsplit(x, "\n")))
    lab <- lab[which.max(nchar(lab))]
    if (length(lab) < 1L) lab <- ""
    return(lab)
  }
  nstr <- maxstr(node_party(obj))
  if (nchar(nstr) < 6) nstr <- "aAAAAa"
  rval <- function(node) {
    node_vp <- viewport(x = unit(0.5, "npc"), y = unit(0.5, "npc"), width = unit(1, "strwidth", nstr) * 1.3,
                        height = unit(3, "lines"), name = paste("node_inner", id_node(node), sep = ""),
                        gp = gpar(fontsize = 18))
    pushViewport(node_vp)
    xell <- c(seq(0, 0.2, by = 0.01), seq(0.2, 0.8, by = 0.05), seq(0.8, 1, by = 0.01))
    yell <- sqrt(xell * (1 - xell))
    lab <- extract_label(node)
    grid.polygon(x = unit(c(xell, rev(xell)), "npc"), y = unit(c(yell, -yell) + 0.5, "npc"), gp = gpar(fill = "white"))
    grid.text(lab[1L], y = unit(1.0 + 0.5 * (lab[2L] != ""), "lines"))
    if (lab[2L] != "") grid.text(lab[2L], y = unit(1, "lines"))
    
    nodeIDvp <- viewport(x = unit(0.5, "npc"), y = unit(1, "npc"),
                         width = max(unit(.6, "lines"), unit(0.9, "strwidth", nam[id_node(node)])),
                         height = max(unit(.6, "lines"), unit(0.9, "strheight", nam[id_node(node)])),
                         gp = gpar(fontsize = 12))
    pushViewport(nodeIDvp)
    grid.rect(gp = gpar(fill = "white"))
    grid.text(nam[id_node(node)])
    popViewport()
    
    upViewport()
  }
  return(rval)
}
class(node_inner_fig1) <- "grapcon_generator"

edge_simple_fig1 <- function(obj) {
  meta <- obj$data
  justfun <- function(i, split) {
    k <- length(split)
    rval <- rep.int(0, k)
    unit(0.5, "npc") + unit(rval[i], "lines")
  }
  function(node, i){
    split <- character_split(split_node(node), meta, digits = 3)$levels
    y <- justfun(i, split)
    split <- split[i]
    if (any(grep(">", split) > 0) | any(grep("<", split) > 0)) {
      tr <- suppressWarnings(try(parse(text = paste("phantom(0)", split)), silent = TRUE))
      if (!inherits(tr, "try-error")) split <- tr
    }
    grid.rect(x = .5 + 0.007, y = y, gp = gpar(fill = "white", col = 0), width = unit(0.88, "strwidth", split))
    grid.text(split, y = y, just = "center", gp = gpar(fontsize = 13))
  }
}
class(edge_simple_fig1) <- "grapcon_generator"


edge_simple_fig2_3 <- function(obj) {return(function(node, i){})}
class(edge_simple_fig2_3) <- "grapcon_generator"

node_inner_fig2 <- function (obj, cutpts) {
  meta <- obj$data
  nam <- names(obj)
  extract_label <- function(node) {
    if (is.terminal(node)) return(rep.int("", 2L))
    varlab <- character_split(split_node(node), meta)$name
    return(c(varlab, ""))
  }
  maxstr <- function(node) {
    lab <- extract_label(node)
    klab <- if (is.terminal(node)) "" else unlist(lapply(kids_node(node), maxstr))
    lab <- c(lab, klab)
    lab <- unlist(lapply(lab, function(x) strsplit(x, "\n")))
    lab <- lab[which.max(nchar(lab))]
    if (length(lab) < 1L) lab <- ""
    return(lab)
  }
  nstr <- maxstr(node_party(obj))
  if (nchar(nstr) < 6) nstr <- "aAAAAa"
  rval <- function(node) {
    node_vp <- viewport(x = unit(0.5, "npc"), y = unit(0.5, "npc"),
                        width = unit(1, "strwidth", nstr) * 1.3, height = unit(3, "lines"),
                        name = paste("node_inner", id_node(node), sep = ""),
                        gp = gpar(fontsize = 15))
    pushViewport(node_vp)
    xell <- c(seq(0, 0.2, by = 0.01), seq(0.2, 0.8, by = 0.05), seq(0.8, 1, by = 0.01))
    yell <- sqrt(xell * (1 - xell))
    lab <- extract_label(node)
    grid.polygon(x = unit(c(xell, rev(xell)), "npc"), y = unit(c(yell, -yell) + 0.5, "npc"),
                 gp = gpar(fill = "white"))
    grid.text(lab[1L], y = unit(1.8 + 0.5 * (lab[2L] != ""), "lines"))
    printCut <- cutpts[names(cutpts) == id_node(node)]
    if (substr(printCut,1,1) == ">"){
      printCut <- substr(printCut,2,nchar(printCut))
      grid.text(bquote(phantom(0)[""> .(printCut)]), x = unit(0.42, "npc"), y = unit(0.85 + 0.5 * (lab[2L] != ""), "lines"), gp=gpar(fontsize=22))
    } else if (substr(printCut,1,1) != "") {
      printCut <- substr(printCut,2,nchar(printCut))
      grid.text(bquote(phantom(0)[""<= .(printCut)]), x = unit(0.42, "npc"), y = unit(0.85 + 0.5 * (lab[2L] != ""), "lines"), gp=gpar(fontsize=22))
    }
    if (lab[2L] != "") grid.text(lab[2L], y = unit(1, "lines"))
    upViewport()
  }
  return(rval)
}
class(node_inner_fig2) <- "grapcon_generator"

node_terminal_fig2 <- function(obj, ns, surv) {
  nam <- names(obj)
  
  extract_label <- function(node) formatinfo_node(node, FUN = NULL, default = c("", ""))
  
  maxstr <- function(node) {
    lab <- extract_label(node)
    klab <- if (is.terminal(node)) "" else unlist(lapply(kids_node(node), maxstr))
    lab <- c(lab, klab)
    lab <- unlist(lapply(lab, function(x) strsplit(x, "\n")))
    return(lab[which.max(nchar(lab))])
  }
  
  nstr <- maxstr(node_party(obj))
  
  rval <- function(node) {
    lab <- extract_label(node)
    height <- length(lab) + 1L
    node_vp <- viewport(x = unit(0.5, "npc"),
                        y = unit(0.5, "npc"),
                        just = c("center", "center"),
                        width = unit(1, "strwidth", nstr) * 1.1,
                        height = unit(height, "lines"),
                        name = paste("node_terminal", id_node(node), sep = ""),
                        gp = gpar())
    pushViewport(node_vp)
    
    nodeN_vp <- viewport(x = unit(0.5, "npc"), y = unit(0.47, "npc"),
                         width = max(unit(1, "lines"), unit(1.3, "strwidth", nam[id_node(node)])),
                         height = max(unit(1, "lines"), unit(1.3, "strheight", nam[id_node(node)])))
    pushViewport(nodeN_vp)
    grid.text(ns[names(ns) == id_node(node)])
    popViewport()
    
    nodeSurv_vp <- viewport(x = unit(0.5, "npc"), y = unit(0.43, "npc"),
                            width = max(unit(1, "lines"), unit(1.3, "strwidth", nam[id_node(node)])),
                            height = max(unit(1, "lines"), unit(1.3, "strheight", nam[id_node(node)])))
    pushViewport(nodeSurv_vp)
    grid.text(paste0("(",surv[names(surv) == id_node(node)],")"))
    popViewport()
    
    upViewport()
  }
  return(rval)
}
class(node_terminal_fig2) <- "grapcon_generator"


node_inner_fig3 <- function (obj, cutpts) {
  meta <- obj$data
  nam <- names(obj)
  extract_label <- function(node) {
    if (is.terminal(node)) return(rep.int("", 2L))
    varlab <- character_split(split_node(node), meta)$name
    return(c(varlab, ""))
  }
  maxstr <- function(node) {
    lab <- extract_label(node)
    klab <- if (is.terminal(node)) "" else unlist(lapply(kids_node(node), maxstr))
    lab <- c(lab, klab)
    lab <- unlist(lapply(lab, function(x) strsplit(x, "\n")))
    lab <- lab[which.max(nchar(lab))]
    if (length(lab) < 1L) lab <- ""
    return(lab)
  }
  nstr <- maxstr(node_party(obj))
  if (nchar(nstr) < 6) nstr <- "aAAAAa"
  rval <- function(node) {
    node_vp <- viewport(x = unit(0.5, "npc"), y = unit(0.5, "npc"),
                        width = unit(1, "strwidth", nstr) * 1.3, height = unit(3, "lines"),
                        name = paste("node_inner", id_node(node), sep = ""),
                        gp = gpar(fontsize = 15))
    pushViewport(node_vp)
    xell <- c(seq(0.15, 0.2, by = 0.01), seq(0.2, 0.8, by = 0.05), seq(0.8, .85, by = 0.01))
    yell <- sqrt(xell * (1 - xell))
    lab <- extract_label(node)
    grid.circle(x = .5, y = .5, gp = gpar(fill = "white"))
    grid.text(lab[1L], y = unit(1.8 + 0.5 * (lab[2L] != ""), "lines"))
    printCut <- cutpts[names(cutpts) == id_node(node)]
    if (substr(printCut,1,1) == ">"){
      printCut <- substr(printCut,2,nchar(printCut))
      grid.text(bquote(phantom(0)[""> .(printCut)]), x = unit(0.42, "npc"), y = unit(0.85 + 0.5 * (lab[2L] != ""), "lines"), gp=gpar(fontsize=22))
    } else if (substr(printCut,1,1) != "") {
      printCut <- substr(printCut,2,nchar(printCut))
      grid.text(bquote(phantom(0)[""<= .(printCut)]), x = unit(0.42, "npc"), y = unit(0.85 + 0.5 * (lab[2L] != ""), "lines"), gp=gpar(fontsize=22))
    }
    
    
    if (lab[2L] != "") grid.text(lab[2L], y = unit(1, "lines"))
    upViewport()
  }
  return(rval)
}
class(node_inner_fig3) <- "grapcon_generator"

node_terminal_fig3 <- function(obj, ns, surv) {
  nam <- names(obj)
  
  extract_label <- function(node) formatinfo_node(node, FUN = NULL, default = c("", ""))
  
  maxstr <- function(node) {
    lab <- extract_label(node)
    klab <- if (is.terminal(node)) "" else unlist(lapply(kids_node(node), maxstr))
    lab <- c(lab, klab)
    lab <- unlist(lapply(lab, function(x) strsplit(x, "\n")))
    return(lab[which.max(nchar(lab))])
  }
  
  nstr <- maxstr(node_party(obj))
  
  rval <- function(node) {
    lab <- extract_label(node)
    height <- length(lab) + 1L
    node_vp <- viewport(x = unit(0.5, "npc"),
                        y = unit(0.5, "npc"),
                        just = c("center", "center"),
                        width = unit(1, "strwidth", nstr) * 1.1,
                        height = unit(height, "lines"),
                        name = paste("node_terminal", id_node(node), sep = ""),
                        gp = gpar())
    pushViewport(node_vp)
    
    nodeN_vp <- viewport(x = unit(0.5, "npc"), y = unit(0.47, "npc"),
                         width = max(unit(1, "lines"), unit(1.3, "strwidth", nam[id_node(node)])),
                         height = max(unit(1, "lines"), unit(1.3, "strheight", nam[id_node(node)])),
                         gp = gpar(fontsize = 9))
    pushViewport(nodeN_vp)
    grid.text(ns[names(ns) == id_node(node)])
    popViewport()
    
    nodeSurv_vp <- viewport(x = unit(0.5, "npc"), y = unit(0.44, "npc"),
                            width = max(unit(1, "lines"), unit(1.3, "strwidth", nam[id_node(node)])),
                            height = max(unit(1, "lines"), unit(1.3, "strheight", nam[id_node(node)])),
                            gp = gpar(fontsize = 8))
    pushViewport(nodeSurv_vp)
    grid.text(paste0("(",surv[names(surv) == id_node(node)],")"))
    popViewport()
    
    upViewport()
  }
  return(rval)
}
class(node_terminal_fig3) <- "grapcon_generator"


